; BDEV Bend Developer [Version 2.2, 9/10/98]
;
; Copyright 1996 - 1998 ManuSoft
;
; ***************************************
; ****  Author:  Owen Wengerd        ****
; ****                               ****
; ****  ManuSoft                     ****
; ****  P.O. Box 84                  ****
; ****  Fredericksburg, OH  44627    ****
; ****  330-695-5903 (Voice)         ****
; ****  330-695-5903 (Fax)           ****
; ****  owenw@manusoft.com           ****
; ***************************************
;
; Bend Development Tool
;   Uses the k-factor method with the
;   specified k-factor and metal thickness
;   to calculate the developed length of
;   a part by selecting the lines and
;   arcs from a cross-section of the part.


;Declare Global Variables
;*BA       ;Bend angle (in degrees)
;*IBR      ;Inside bend radius
;*KF       ;k-Factor
;*TH       ;Metal thickness

;Define the application
(defun C:BDEV
  (/

  ;Declare Local Variables
  devlen   ;Total developed length
  entl     ;Entity dxf list
  inp      ;Result of (getkword)
  olderr   ;Original error handler
  sel      ;Result of (entget)

  ;Declare Local Functions
  adddev   ;Add and display developed length
  dxf      ;Extract dxf code
  errexit  ;Error handler
  numchk   ;Validate number
  numinp   ;Input valid number
  )

  ;Define Local Functions

  ;============================================
  ;= (adddev <length_to_add>)                 =
  ;=   adds <length_to_add> to 'devlen' and   =
  ;=   prints the new total.                  =
  ;============================================
  (defun adddev (addlen)
    (princ
      (strcat
        "\nTotal developed length is "
        (rtos
          (setq devlen (+ devlen addlen))
          (getvar "LUNITS")
          (+ 2 (getvar "LUPREC"))))))

  ;============================================
  ;= (dxf <dxf_code> <entity_list>)           =
  ;=   extracts the value of the <dxf_code>   =
  ;=   group from <entity_list>.              =
  ;============================================
  (defun dxf (grp elist)
    (cdr (assoc grp elist)))

  ;============================================
  ;= (errexit <message>)                      =
  ;=   processes error <message>.             =
  ;============================================
  (defun errexit (s)
    (setq *error* olderr)
    (if (and *error* (listp *error*))
      (*error* s)
      (if
        (not
          (member s
            '("console break"
              "Function cancelled"
              "quit / exit abort")))
	      (princ (strcat "\nError:  " s))))
    (princ))

  ;============================================
  ;= (numchk <number>)                        =
  ;=   returns T if <number> is a positive    =
  ;=   number or zero.                        =
  ;============================================
  (defun numchk (num)
    (and (numberp num) (>= num 0)))

  ;============================================
  ;= (numinp <prompt> <default>)              =
  ;=   gets input and validates it with       =
  ;=   (numchk) before returning it.          =
  ;============================================
  (defun numinp (prompt default / inp)
    (while
      (and
        (setq inp
          (getdist
            (strcat
              "\n"
              prompt
              "<"
              (rtos
                default
                (getvar "LUNITS")
                (+ 2 (getvar "LUPREC")))
              ">: ")))
        (not (numchk inp))))
    (cond (inp) (default)))


  ;********************************************
  ;*              Main Program                *
  ;********************************************

  ;set up our error handler
  (setq olderr *error* *error* errexit)

  ;check global variables for valid values
  ;and set to defaults if necessary
  (if (not (numchk *TH))
    (setq *TH 0.0598))

  ;get input and store in global variables
  (setq *TH
        (numinp "Material Thickness" *TH)
        *KF ;set k-Factor to 40%
        0.40)    ;[or, could use (numinp)]

  ;reset the running blank length total
  (setq devlen 0)

  ;loop until the user is finished selecting
  (while
    (or
      (setq sel (entsel "\nPick a segment: "))
      (/= (getvar "ERRNO") 52))
    (cond
      ( (not sel)
        (princ "  <Nothing selected>"))
      ( (= "LINE"
          (dxf 0
            (setq entl (entget (car sel)))))
        (adddev
          (distance
            (dxf 10 entl)
            (dxf 11 entl))))
      ( (= "ARC" (dxf 0 entl))
        (initget "Inside Outside")
        (setq inp
          (getkword
            (strcat
              "\nIs this arc ("
              "<Inside>/Outside"
              ")?  ")))
        (setq *IBR
          (if (= inp "Outside")
            (- (dxf 40 entl) *TH)
            (dxf 40 entl)))
        (setq *BA
          (- (dxf 51 entl) (dxf 50 entl)))
        (if (< *BA 0)
          (setq *BA (+ *BA (* pi 2))))
        (adddev (* *BA (+ *IBR (* *KF *TH))))
        (setq *BA (/ (* *BA 180) pi)))
      ( (princ
          (strcat
            "  <Invalid>"
            "\nYou must select "
            "an ARC or LINE entity.")))))
   (princ))
;End-of-file
